package com.marvinelsen.willow

import com.marvinelsen.willow.domain.SearchMode
import com.marvinelsen.willow.domain.SqliteDictionary
import com.marvinelsen.willow.domain.objects.DictionaryEntry
import com.marvinelsen.willow.ui.undo.Command
import com.marvinelsen.willow.ui.util.ClipboardHelper
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch

@Suppress("TooManyFunctions")
class Interactor(
    private val model: Model,
    private val dictionary: SqliteDictionary,
) {
    private val coroutineScope = MainScope()

    fun copyHeadwordOfSelectedEntry() {
        model.selectedEntry?.let { ClipboardHelper.copyString(it.traditional) }
    }

    fun copyPronunciationOfSelectedEntry() {
        model.selectedEntry?.let { ClipboardHelper.copyString(it.pinyinWithToneMarks) }
    }

    fun undoSelection() {
        model.undoManager.undo()
    }

    fun redoSelection() {
        model.undoManager.redo()
    }

    fun search(query: String, searchMode: SearchMode) {
        coroutineScope.launch {
            model.isSearching = true
            model.searchResults.setAll(dictionary.search(query, searchMode))
            model.isSearching = false
        }
    }

    fun findWordsBeginning() {
        coroutineScope.launch {
            model.isFindingWordsBeginning = true
            model.wordsBeginning.setAll(
                model.selectedEntry?.let {
                    dictionary
                        .findWordsBeginningWith(it)
                }
            )
            model.isFindingWordsBeginning = false
            model.finishedFindingWordsBeginning = true
        }
    }

    fun findWordsContaining() {
        coroutineScope.launch {
            model.isFindingWordsContaining = true
            model.wordsContaining.setAll(
                model.selectedEntry?.let {
                    dictionary
                        .findWordsContaining(it)
                }
            )
            model.isFindingWordsContaining = false
            model.finishedFindingWordsContaining = true
        }
    }

    fun findCharacters() {
        coroutineScope.launch {
            model.isFindingCharacters = true
            model.characters.setAll(
                model.selectedEntry?.let {
                    dictionary
                        .findCharactersOf(it)
                }
            )
            model.isFindingCharacters = false
            model.finishedFindingCharacters = true
        }
    }

    fun findSentences() {
        coroutineScope.launch {
            model.isFindingSentences = true
            model.sentences.setAll(
                model.selectedEntry?.let {
                    dictionary
                        .findExampleSentencesFor(it)
                }
            )
            model.isFindingSentences = false
            model.finishedFindingSentences = true
        }
    }

    fun deepDive(entry: DictionaryEntry) {
        model.undoManager.execute(object : Command {
            private val previouslySelectedEntry = model.selectedEntry

            override fun execute() {
                select(entry)
            }

            override fun undo() {
                if (previouslySelectedEntry == null) return

                select(previouslySelectedEntry)
            }
        })
    }

    fun normalSelect(entry: DictionaryEntry) {
        model.undoManager.reset()
        select(entry)
    }

    private fun select(entry: DictionaryEntry) {
        model.finishedFindingCharacters = false
        model.finishedFindingWordsBeginning = false
        model.finishedFindingWordsContaining = false
        model.finishedFindingSentences = false

        model.selectedEntry = entry
    }
}
